#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/input.h>
#include <linux/uinput.h>
#include <signal.h>

#define X_RES 2000
#define Y_RES 2000
#define MAX_FINGERS 10

int event(int fd, int type, int code, int value)
{
	struct input_event ev;

	memset(&ev, 0, sizeof(struct input_event));
	ev.type = type;
	ev.code = code;
	ev.value = value;

	if (write(fd, &ev, sizeof(struct input_event)) < 0)
		return 0;
	return 1;
}

int do_input(int fd)
{
	int x, y;

	/* let's poke everything in a 10/10 grid */
	for  (x = 0; x < X_RES; x += 10)
		for (y = 0; y < Y_RES; y += 10) {
			event(fd, EV_ABS, ABS_MT_SLOT, 0);
			event(fd, EV_ABS, ABS_MT_TRACKING_ID, 1);
			event(fd, EV_ABS, ABS_MT_POSITION_X, x);
			event(fd, EV_ABS, ABS_MT_POSITION_Y, y);
			event(fd, EV_SYN, SYN_REPORT, 0);
			event(fd, EV_ABS, ABS_MT_TRACKING_ID, -1);
			event(fd, EV_SYN, SYN_REPORT, 0);
			usleep(10000);
		}
	return 1;
}

int main(int argc, char **argv)
{
	sigset_t sigset;
	struct uinput_user_dev ud;
	int ret, fd, sig;

	fd = open("/dev/uinput", O_WRONLY);
	if (fd < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_EVBIT, EV_ABS);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_EVBIT, EV_KEY);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_KEYBIT, BTN_TOUCH);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_EVBIT, EV_SYN);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_X);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_Y);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_MT_POSITION_X);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_MT_POSITION_Y);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_MT_PRESSURE);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_MT_TRACKING_ID);
	if (ret < 0)
		return EXIT_FAILURE;

	ret = ioctl(fd, UI_SET_ABSBIT, ABS_MT_SLOT);
	if (ret < 0)
		return EXIT_FAILURE;

	memset(&ud, 0, sizeof(ud));
	sprintf(ud.name, "Event Testing Device");
	ud.id.bustype = BUS_USB;
	ud.id.vendor = 1234;
	ud.id.product = 5678;
	ud.id.version = 1;
	ud.absmax[ABS_MT_SLOT] = MAX_FINGERS;
	ud.absmax[ABS_X] = X_RES;
	ud.absmax[ABS_Y] = Y_RES;
	ud.absmax[ABS_MT_POSITION_X] = X_RES;
	ud.absmax[ABS_MT_POSITION_Y] = Y_RES;
	ud.absmax[ABS_MT_TOUCH_MAJOR] = 1;
	ud.absmax[ABS_MT_PRESSURE] = 10;
	ud.absfuzz[ABS_MT_POSITION_X] = 2;
	ud.absfuzz[ABS_MT_POSITION_Y] = 2;
	ret = write(fd, &ud, sizeof(ud));
	if (ret < 0)
		return EXIT_FAILURE;
	ret = ioctl(fd, UI_DEV_CREATE);
	if (ret < 0)
		return EXIT_FAILURE;

	sigemptyset(&sigset);
	sigaddset(&sigset, SIGUSR1);
	sigprocmask(SIG_BLOCK, &sigset, NULL);
	sigwait(&sigset, &sig);

	do_input(fd);

	return EXIT_SUCCESS;
}
